package org.jboss.cache.search;

import org.jboss.cache.search.CacheEntityLoader;
import org.hibernate.search.engine.DocumentExtractor;
import org.hibernate.search.engine.SearchFactoryImplementor;
import org.apache.lucene.search.Hits;
import org.apache.lucene.search.IndexSearcher;

import java.util.NoSuchElementException;
import java.io.IOException;

/**
 * @author Navin Surtani (<a href="mailto:nsurtani@redhat.com">nsurtani@redhat.com</a>)
 */
public class LazyQueryResultIterator implements QueryResultIterator
{
   private int index = 0;
   private int max = 0;
   private int first = 0;
   private CacheEntityLoader entityLoader;
   private DocumentExtractor extractor;
   private Hits hits;
   private IndexSearcher searcher;
   private SearchFactoryImplementor searchFactory;

   public LazyQueryResultIterator(DocumentExtractor extractor, CacheEntityLoader entityLoader, Hits hits,
                                  IndexSearcher searcher, SearchFactoryImplementor searchFactory, int first, int max)
   {
      this.extractor = extractor;
      this.entityLoader = entityLoader;
      index = first;
      this.first = first;
      this.max = max;
      this.hits = hits;
      this.searcher = searcher;
      this.searchFactory = searchFactory;
   }

   public void jumpToResult(int index) throws IndexOutOfBoundsException
   {
      if (index < first || index > max)
      {
         throw new IndexOutOfBoundsException("The given index is incorrect. Please check and try again.");
      }

      this.index = first + index;

   }

   public void first()
   {
      index = first;
   }

   public void last()
   {
      index = max;
   }

   public void afterFirst()
   {
      index = first + 1;
   }

   public void beforeLast()
   {
      index = max - 1;
   }

   public boolean isFirst()
   {
      return index == first;
   }

   public boolean isLast()
   {
      return index == max;
   }

   public boolean isAfterFirst()
   {
      return index == first + 1;
   }

   public boolean isBeforeLast()
   {
      return index == max - 1;
   }

   public void close()
   {
      IndexSearcherCloser.closeSearcher(searcher, searchFactory.getReaderProvider());
   }

   public boolean hasNext()
   {
      return index <= max;
   }

   public Object next()
   {
      Object toReturn = null;
      try
      {
         String documentId = (String) extractor.extract(hits, index).id;
         CacheEntityId id = new CacheEntityId(documentId);
         toReturn = entityLoader.load(id);
         index++;
      }
      catch (IOException e)
      {
         e.printStackTrace();
      }

      return toReturn;
   }

   public boolean hasPrevious()
   {
      return index >= first;
   }

   public Object previous()
   {
      Object toReturn = null;
      try
      {
         String documentId = (String) extractor.extract (hits, index).id;
         CacheEntityId id = new CacheEntityId(documentId);
         toReturn = entityLoader.load(id);
         index--;
      }
      catch (IOException e)
      {
         e.printStackTrace();
      }
      return toReturn;
   }

   public int nextIndex()
   {
      if (!hasNext()) throw new NoSuchElementException("Out of boundaries");
      return index + 1;
   }

   public int previousIndex()
   {
      if (!hasPrevious()) throw new NoSuchElementException("Out of boundaries.");
      return index - 1;
   }

   /**
    * This method is not supported and should not be used. Use cache.remove() instead.
    */
   public void remove()
   {
      throw new UnsupportedOperationException("Not supported as you are trying to change something in the cache");
   }

   /**
    * This method is not supported in and should not be called. Use cache.put() instead.
    *
    * @param o
    * @throws UnsupportedOperationException
    */
   public void set(Object o) throws UnsupportedOperationException
   {
      throw new UnsupportedOperationException("Not supported as you are trying to change something in the cache");
   }

   /**
    * This method is not supported in and should not be called. Use cache.put() instead.
    *
    * @param o
    * @throws UnsupportedOperationException
    */

   public void add(Object o)
   {
      throw new UnsupportedOperationException("Not supported as you are trying to change something in the cache");
   }
}
